capture log close
set more off
log using pefr, replace text

use pefr, clear

* plot data
generate mean_wm = (wm1+wm2)/2
summ mean_wm
local wm_mean= r(mean)

twoway (scatter wm1 id, msymbol(circle)) ///
       (scatter wm2 id, msymbol(circle_hollow)), ///
       xtitle(Subject id) ytitle(Mini Wright measurements) ///
       legend( order(1 "Occasion 1" 2 "Occasion 2")) yline(`wm_mean')

gen junk2 = wm2+100
local wm_mean2=`wm_mean'+50

twoway (scatter wm1 id, msymbol(circle)) ///
       (scatter junk2 id, msymbol(circle_hollow)), ///
       xtitle(Subject id) ytitle("Mini Wright measurements, adding 100 at Occasion 2") ///
       legend( order(1 "Occasion 1" 2 "Occasion 2")) yline(`wm_mean2')

list if id<6, clean noobs

* stack repeated observations for same method into single variable
reshape long wp wm, i(id) j(occasion)
list if id<6, clean noobs

* estimation using xtreg
xtreg wm, i(id) mle

* estimation using xtmixed
xtmixed wm || id:, mle

* estimation using gllamm
gllamm wm, i(id) nip(12) adapt
estimates store RI

* random intercept model with occasion 2 dummy

gen occ2=occasion==2
xtmixed wm occ2 || id:, mle

* ML estimates of random intercepts using regression
quietly xtmixed wm || id:, mle
predict pred, xb
gen res = wm - pred
quietly tabulate id, gen(subject)
regress res subject1-subject17, nocons

matrix mlest = e(b)'
sort occasion id
svmat mlest
egen mle = mean(mlest), by(id)

* ML estimates of random intercepts using mean residual
egen ml = mean(res), by(id)
sort id

* Shrinkage factor
estimates restore RI
local Rj = [id1]_cons^2/([id1]_cons^2+exp([lns1]_cons)^2/2)
disp `Rj'

matrix list e(b)

* Get EB from ML
gen eb1 = `Rj'*ml

* Get EB using xtmixed
quietly xtmixed wm || id:, mle
predict eb2, reffects

sort id
format eb1 eb2 %8.2f
list id eb1 eb2 if occasion==1, clean noobs


* Get EB using gllapred
estimates restore RI
gllapred eb, u
disp ebs1[1]

* diagnostic standard errors
disp sqrt([id1]_cons^2-ebs1^2)

* conditional standard errors
disp sqrt(`Rj'*(1-`Rj'))*[id1]_cons

log close
exit
