capture log close
log using gcse, replace text
set more off
set scheme sj

/* Least squares regressions for individual schools */
use gcse, clear
regress gcse lrt if school==1
predict p_gcse, xb 
twoway (scatter gcse lrt) (line p_gcse lrt, sort) if school==1, ///
  xtitle(LRT) ytitle(GCSE)

egen num=count(gcse), by(school)
statsby inter=_b[_cons] slope=_b[lrt], by(school) clear: ///
  regress gcse lrt if num>4 
graph twoway scatter slope inter, xtitle(Intercept) ytitle(Slope)

summarize inter slope
corr inter slope, cov

sort school
merge school using http://www.stata-press.com/data/mlmus/gcse
drop _merge
gen pred = inter + slope*lrt

sort school lrt
graph twoway (line pred lrt, connect(ascending)), xtitle(LRT) ///
   ytitle(Fitted regression lines)


/* Models using xtmixed */
xtmixed gcse lrt || school:, mle
estimates store model1

xtmixed gcse lrt || school: lrt,  cov(unstr) mle
estimates store model2
lrtest model1 model2

predict reff*, reffects
gen inter_eb = reff2 + _b[_cons]
gen slope_eb = reff1 + _b[lrt]

by school, sort: gen f = _n==1
list school inter inter_eb slope slope_eb if f==1&school<=10 

twoway (scatter inter_eb inter if f==1, mlabel(school)) ///
  (function y=x, range(-10 10)), legend(off) xtitle(ML estimate) ///
  ytitle(EB prediction) legend(off)

twoway (scatter slope_eb slope if f==1, mlabel(school)) ///
  (function y=x, range(0 1.5)), xtitle(ML estimate) ///
  ytitle(EB prediction) legend(off)

predict prmodel2, fitted
sort school lrt
graph twoway (line prmodel2 lrt, connect(ascending)), xtitle(LRT) ///
  ytitle(Empirical Bayes regression lines for model 2)

estimates restore model1
predict prmodel1, fitted
sort school lrt
graph twoway (line prmodel1 lrt, connect(ascending)), xtitle(LRT) ///
  ytitle(Empirical Bayes regression lines for model 1)

  
/* Same models using gllamm */
drop reff1 reff2 inter_eb slope_eb prmodel1 prmodel2

gllamm gcse lrt, i(school) adapt
estimates store model1

eq slope: lrt
gen cons=1
eq inter: cons

matrix a=e(b)
matrix a=(a,0,0)
gllamm gcse lrt, i(school) nrf(2) eqs(inter slope) ip(m) nip(15) ///
  adapt from(a) copy
estimates store model2
lrtest model1 model2

gllapred reff, u

gen inter_eb = reffm1 + _b[_cons]
gen slope_eb = reffm2 + _b[lrt]

gllapred prmodel2, linpred

gsort +reffm1 -f
gen rank=sum(f)
gen upper = reffm1+1.96*reffs1+.4
serrbar reffm1 reffs1 rank if f==1, addplot(scatter upper rank,  ///
  mlabel(school) msymbol(none) mlabpos(0)) scale(1.96) xtitle(Rank)  ///
  ytitle(Prediction) legend(off)

log close
exit
